import qrcode


class AssetQRCode(qrcode.QRCode):
    """An AssetQRCode instance generates and displays a QR code. At the time
    of writing this. The QR code is built only by being passed a list of string
     objects using the add_data() method"""

    def __init__(self, instructions):
        self.qr = qrcode.QRCode(error_correction=qrcode.constants.ERROR_CORRECT_L)
        super(AssetQRCode, self).__init__()
        self.build(instructions)

    def build(self, instructions):
        for step in instructions.instruction_steps():
            self.qr.add_data(step)
        return self.qr

    def display(self):
        return self.qr.make_image().show()



import json
import shlex
import subprocess

BASE_COMMAND = '/usr/local/bin/cfgutil --foreach --format JSON get IMEI serialNumber deviceType totalDiskCapacity'


def parse_cfgutil_output(output):
    return json.loads(output)


def get_device_properties_from_cfgutil_output(cfgutil_output=None):
    cfgutil_output = cfgutil_output or _get_output_of_cfgutil_command()
    return parse_cfgutil_output(cfgutil_output)


def get_hardware_properties_for_attached_devices(device_properties=None):
    device_properties = device_properties or get_device_properties_from_cfgutil_output()
    return [device_properties['Output'][ecid] for ecid in device_properties['Devices']]


def _get_output_of_cfgutil_command(arguments=None):
    arguments = arguments or ''
    full_command = shlex.split(' '.join([BASE_COMMAND, arguments]))
    return subprocess.check_output(full_command).decode('utf-8')



import re
import yaml
from inqry.system_specs import diskutil


def create_from_diskutil_output(output):
    return Disk(yaml.load(output))


def create_disks_from_physical_disk_ids():
    return [create_from_diskutil_output(diskutil.get_disk_info(disk_id)) for disk_id in diskutil.get_all_physical_ids()]


def get_internal_storage():
    internal_disks = create_disks_from_physical_disk_ids()
    return [disk for disk in internal_disks if disk.is_internal and not disk.is_virtual()]


class Disk:
    def __init__(self, attributes=None):
        self.attributes = attributes or {}

    @property
    def device_location(self):
        return self.attributes.get('Device Location')

    @property
    def removable_media(self):
        return self.attributes.get('Removable Media')

    @property
    def is_fixed(self):
        return self.removable_media == 'Fixed' or not self.removable_media

    @property
    def is_internal(self):
        return self.device_location != 'External'

    def is_virtual(self):
        return self.attributes.get('Virtual')

    @property
    def is_external(self):
        return self.device_location == 'External' or self.is_fixed is False

    @property
    def type(self):
        if self.is_ssd:
            return 'SSD'
        elif not self.is_ssd:
            return 'HDD'
        else:
            return 'Unknown'

    @property
    def device_name(self):
        return self.attributes.get('Device / Media Name')

    @property
    def is_ssd(self):
        return self.attributes.get('Solid State')

    @property
    def verbose_disk_size(self):
        return self.attributes.get('Disk Size') or self.attributes.get('Total Size')

    @property
    def size(self):
        disk_size_pattern = re.compile(r'(?P<disk_size>\d+\.?\d* [MGT]?B) .*$')
        disk_size_match = re.match(disk_size_pattern, self.verbose_disk_size)
        if disk_size_match:
            return disk_size_match.group('disk_size')



from enum import Enum
import re
import subprocess
import yaml

BASE_COMMAND = '/usr/sbin/system_profiler'


class DataTypes(Enum):
    def __str__(self):
        return 'SP' + self.value + 'DataType'

    DEVELOPER_TOOLS = 'DeveloperTools'
    DIAGNOSTICS = 'Diagnostics'
    DISPLAYS = 'Displays'
    HARDWARE = 'Hardware'
    MEMORY = 'Memory'
    SOFTWARE = 'Software'
    STORAGE = 'Storage'
    THUNDERBOLT = 'Thunderbolt'


def get_data(data_type):
    """
    Returns system_profiler data on macOS as a dictionary

    `data_type` must be one of `DataTypes`, i.e.

        get_data(DataTypes.HARDWARE)

    :param data_type:
    :type data_type: DataTypes
    :return dict:
    """
    return parse_command_output(_get_command_output(str(data_type)))


def parse_command_output(output):
    return yaml.load(_yamlize_output(output))


def _get_command_output(*arguments):
    return subprocess.check_output([BASE_COMMAND] + list(arguments)).decode('utf-8')


def _yamlize_output(output):
    return '\n'.join([_yamlize_line(line) for line in output.split('\n')])


def _yamlize_line(line):
    line = _remove_dash_after_first_colon(line)

    eight_spaces = '        '
    line = line[2:] if line.startswith(eight_spaces) else line

    return line


def _remove_dash_after_first_colon(line):
    return re.sub(pattern=r'(.+: )-', repl=r'\1', string=line)


def get_hardware_overview():
    return get_data(DataTypes.HARDWARE)['Hardware']['Hardware Overview']



__author__ = "Microsoft Apex Lab"
__copyright__ = "Copyright 2017, Microsoft Corporation"
__credits__ = ["Eric Hanko", "Jacob Zaval", "Michael Brown", "Andre Shields", "Ryan Dominguez", "Eammon Hanlon"]
__license__ = "MIT"
__email__ = "apxlab@microsoft.com"




from tkinter import *
import subprocess
from inqry.asset_qrcode import AssetQRCode
from inqry.system_specs.systemspecs import SystemSpecs
from inqry.system_specs import ios_system_profiler
from inqry.form_instructions import FormInstructions


def devices_are_attached():
    try:
        return subprocess.check_output(['/usr/local/bin/cfgutil', 'list'])
    except FileNotFoundError:
        print('No such file or directory: "/usr/local/bin/cfgutil"')
        return False


def mobile_capability():
    return 'active' if devices_are_attached() else 'disabled'


class InQRyGUI:
    def __init__(self):
        self.root_window = Tk()
        self.root_window.title('InQRy')
        self.form_factor = StringVar()
        self.new_model_selected = IntVar()
        self.alias_entry = Entry(self.root_window)
        self.alias_entry.grid(row=1, column=1, pady=5)
        self.systemspecs = SystemSpecs()
        self.mobile_capability = mobile_capability()
        self.create_widgets()

    def click(self):
        if self.new_model_selected.get():
            form_instructions = FormInstructions(self.systemspecs, 'New Model')
            AssetQRCode(form_instructions).display()
        elif self.form_factor.get() == 'Mobile':
            for devicespecs in ios_system_profiler.get_hardware_overview():
                form_instructions = FormInstructions(devicespecs, self.form_factor.get(), self.alias_entry.get())
                AssetQRCode(form_instructions).display()
        else:
            form_instructions = FormInstructions(self.systemspecs, self.form_factor.get(), self.alias_entry.get())
            AssetQRCode(form_instructions).display()

    def create_widgets(self):
        generate_qr_button = Button(self.root_window, text='Generate QR Code', command=self.click)
        generate_qr_button.grid(row=6, columnspan=2, pady=15)

        desktop_radio_button = Radiobutton(
            self.root_window, text='Desktop', variable=self.form_factor, value='Desktop')
        desktop_radio_button.select()
        desktop_radio_button.grid(row=2, columnspan=2)

        portable_radio_button = Radiobutton(
            self.root_window, text='Portable', variable=self.form_factor, value='Portable')
        portable_radio_button.deselect()
        portable_radio_button.grid(row=3, columnspan=2)

        portable_radio_button = Radiobutton(
            self.root_window, text='Mobile', variable=self.form_factor, value='Mobile', state=self.mobile_capability)
        portable_radio_button.deselect()
        portable_radio_button.grid(row=4, columnspan=2)

        alias_label = Label(self.root_window, text='Alias:')
        alias_label.grid(row=1, column=0, pady=5, sticky=E)

        new_model_checkbutton = Checkbutton(
            self.root_window, text='New Model', variable=self.new_model_selected)
        new_model_checkbutton.deselect()
        new_model_checkbutton.grid(row=0, columnspan=2)


if __name__ == '__main__':
    gui = InQRyGUI()
    gui.root_window.mainloop()



import sys

OS = sys.platform

if OS == 'win32':
    from inqry.system_specs import windows_system_profiler as system_profiler
    from inqry.system_specs import windisk as diskutility
elif OS == 'darwin':
    from inqry.system_specs import mac_system_profiler as system_profiler
    from inqry.system_specs import macdisk as diskutility
else:
    raise OSError('Operating system unknown')


def _human_readable(component):
    size = round(int(component) / 10 ** 9)
    if size >= 1000:
        return str(size / 1000) + " TB"
    else:
        return str(size) + " GB"



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

long_description="""
InQRy is a cross-platform application that generates a single QR code containing the machine's hardware specifications.
This application is designed primarily to be used during a physical inventory procedure.

The QR code contains detailed information about the client machine or device, which can then be scanned it quickly add
assets into a Snipe-IT database.
"""

setup(
    name='inqry',
    author=['Microsoft Apex Lab'],
    author_email='apxlab@microsoft.com',
    description='A cross-platform utility used to generate a QR code containing hardware specs',
    license='MIT',
    long_description=long_description,
    packages=['inqry', 'inqry.system_specs'],
    install_requires=['Pillow',
                      'pytest',
                      'PyYAML',
                      'qrcode',
                      'wmi;platform_system=="Windows"',
                      'pypiwin32;platform_system=="Windows"'],
    tests_require=['pytest'],
    url='https://github.com/Microsoft/InQRy',
    version='1.2.3',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3 :: Only']
)




import re
import subprocess

BASE_COMMAND = 'C:\\WINDOWS\\system32\\WindowsPowerShell\\v1.0\\powershell.exe Get-PhysicalDisk'


def get_physical_disk_identifiers(unique_id_output=None):
    unique_id_output = unique_id_output or list_all_ids()
    physical_disk_id_pattern = re.compile(r'(?!UniqueId)(\b.+\b|{\b.+\b}|\b.+:.+\b)')
    return re.findall(physical_disk_id_pattern, unique_id_output)


def get_physical_disk_friendly_names(friendly_name_output=None):
    friendly_name_output = friendly_name_output or list_all_friendly_names()
    friendly_name_pattern = re.compile(r'(?!FriendlyName)(\b.+\b)')
    return re.findall(friendly_name_pattern, friendly_name_output)


def get_disk_info(unique_id):
    return _get_output_of_get_physical_disk_command(arguments='-UniqueID \'{}\' | Select *'.format(unique_id))


def get_disk_info_from_friendly_name(friendly_name):
    return _get_output_of_get_physical_disk_command(arguments='-FriendlyName \'{}\' | Select *'.format(friendly_name))


def list_all_ids():
    return _get_output_of_get_physical_disk_command(arguments='"| Select UniqueId"')


def list_all_friendly_names():
    return _get_output_of_get_physical_disk_command(arguments='"| Select FriendlyName"')


def _get_output_of_get_physical_disk_command(arguments=None):
    arguments = arguments or ''
    full_command = ' '.join([BASE_COMMAND, arguments])
    return subprocess.check_output(full_command).decode('utf-8')



class FormInstructions:
    def __init__(self, specs, form_factor=None, user=None):
        self.short_delay = '~d'
        self.long_delay = '~d~d'
        self.tab = '~t'
        self.enter = '~e'
        self.space = '\x20'
        self.status = 'Ready'
        self.model_id = specs.model_identifier
        self.serial = specs.serial_number
        self.fieldset = form_factor or 'Desktop'
        self.user = user or ''
        self.processor = '{} {}'.format(specs.processor_speed, specs.processor_name)
        self.fieldsets = {'Desktop': (self._text_box(self.processor) +
                                      self._text_box(specs.memory) +
                                      self._text_box(specs.drive1) +
                                      self._text_box(specs.drive2) +
                                      self._text_box(specs.drive3) +
                                      self._text_box(specs.drive4)),

                          'Portable': (self._text_box(self.processor) +
                                       self._text_box(specs.memory) +
                                       self._text_box(specs.drive1)),

                          'New Model': (self._text_box(specs.model_name) +
                                        [self.tab,
                                         self.tab,
                                         self.short_delay +
                                         self.model_id]),

                          'Mobile': (self._text_box(specs.imei) +
                                     self._text_box(specs.mobile_storage))}

    def _common_fields(self, unique_fields):
        return (self._list_box(self.model_id) +
                unique_fields +
                self._list_box(self.status) +
                [self.short_delay,
                 self.space,
                 self.short_delay,
                 self.user,
                 self.short_delay,
                 self.enter,
                 self.short_delay,
                 self.tab] +
                [self.short_delay,
                 self.serial])

    def _text_box(self, field_content):
        return [self.short_delay,
                field_content,
                self.tab]

    def _list_box(self, field_content):
        return [self.space,
                self.short_delay,
                field_content,
                self.long_delay,
                self.enter,
                self.long_delay,
                self.tab,
                self.short_delay]

    def instruction_steps(self):
        if self.fieldset == 'New Model':
            return self.fieldsets['New Model']
        else:
            return self._common_fields(self.fieldsets[self.fieldset])



import platform
from inqry.system_specs import diskutility
from inqry.system_specs import system_profiler


class SystemSpecs(object):
    """Represents the machine's system specifications. A SystemSpec instance
     has several machine components that are accessible via simple property
     methods.

    A SystemSpecs object should also be able to be used the same way,
    regardless of which operating system the specs were generated from.

    A SystemSpecs instance is platform agnostic, but does contain the os_type
    attribute"""

    def __init__(self, hardware_overview=None, internal_storage=None, os_type=None):
        self.os_type = os_type or platform.system()
        self.hardware_overview = hardware_overview or system_profiler.get_hardware_overview()
        self.internal_storage = internal_storage or diskutility.get_internal_storage()

    @property
    def model_name(self):
        return self.hardware_overview.get('Model Name')

    @property
    def model_identifier(self):
        return self.hardware_overview.get('Model Identifier')

    @property
    def serial_number(self):
        return self.hardware_overview.get('Serial Number (system)')

    @property
    def processor_name(self):
        return self.hardware_overview.get('Processor Name')

    @property
    def processor_speed(self):
        return self.hardware_overview.get('Processor Speed')

    @property
    def memory(self):
        return self.hardware_overview.get('Memory')

    @property
    def drive_count(self):
        return len(self.storage)

    @property
    def imei(self):
        return self.hardware_overview.get('IMEI')

    def mobile_storage(self):
        return self.hardware_overview.get('Storage')

    @property
    def storage(self):
        return {'Drive {}'.format(disk_count): '{} {} ({})'.format(disk.size, disk.type, disk.device_name) for
                disk_count, disk in enumerate(self.internal_storage, 1)}

    @property
    def drive1(self):
        return self.storage.get('Drive 1')

    @property
    def drive2(self):
        return self.storage.get('Drive 2')

    @property
    def drive3(self):
        return self.storage.get('Drive 3')

    @property
    def drive4(self):
        return self.storage.get('Drive 4')



import re

import wmi


class WindowsProfile:
    """A WindowsProfile instance uses the third-party module "wmi". When instantiated, the instance gets all needed
    attribute data structures from the appropriate WMI classes."""

    def __init__(self):
        c = wmi.WMI()
        self.win32_bios = c.Win32_BIOS()[0]
        self.win32_processor = c.Win32_Processor()[0]
        self.win32_disk_drive = c.Win32_DiskDrive()
        self.win32_computer_system = c.Win32_ComputerSystem()[0]
        self.win32_computer_system_product = c.Win32_ComputerSystemProduct()[0]
        self.win32_physical_memory = c.Win32_PhysicalMemory()[0]

    def get_all_windows_system_components(self):
        return {'Model Name': self._get_model_name(),
                'Model Identifier': self._get_model_identifier(),
                'Manufacturer': self.win32_computer_system.Manufacturer,
                'Serial Number (system)': self._get_serial_number(),
                'Number of Processors': self.win32_computer_system.NumberOfProcessors,
                'Total Number of Cores': self.win32_processor.NumberOfCores,
                'Memory': self.get_memory_in_gigabytes(self.win32_computer_system.TotalPhysicalMemory),
                'Processor Name': self.get_processor_name(self.win32_processor.Name),
                'Processor Speed': self.get_processor_speed(self.win32_processor.Name)}

    @staticmethod
    def _human_readable(component):
        return str(round(int(component) / 1024 ** 3)) + ' GB'

    @staticmethod
    def _split_processor(name):
        pattern = re.compile(r' @ ')
        return re.split(pattern, name)

    def _get_model_name(self):
        return self.win32_computer_system_product.Name or self.win32_computer_system.Model

    def _get_model_identifier(self):
        try:
            return self.win32_computer_system.SystemSKUNumber or self.win32_computer_system_product.Version
        except AttributeError:
            return self._get_model_name()

    def _get_serial_number(self):
        return self.win32_bios.SerialNumber or self.win32_computer_system_product.IdentifyingNumber

    @staticmethod
    def _remove_extra_processor_data(processor):
        pattern = re.compile(r'\(\w\)')
        return re.sub(pattern, '', processor)

    @staticmethod
    def get_processor_name(full_cpu_name):
        return WindowsProfile._remove_extra_processor_data(WindowsProfile._split_processor(full_cpu_name)[0])

    @staticmethod
    def get_processor_speed(full_processor_name):
        return WindowsProfile._split_processor(full_processor_name)[1]

    @staticmethod
    def get_memory_in_gigabytes(memory_bytes):
        return WindowsProfile._human_readable(memory_bytes)


def get_hardware_overview():
    """
    Returns all components from a WindowsProfile instance a dictionary with the same keys as a Mac system profile
    :return: dict:
    """
    return WindowsProfile().get_all_windows_system_components()
